@@ -2,6 +2,7 @@
 
 import asyncio
 import json
+import time
 
 import aiohttp
 from aiohttp.client_exceptions import ClientResponseError
@@ -9,25 +10,28 @@ from yarl import URL
 
 from .exceptions import NotFoundError, TractiveError, UnauthorizedError
 
+CLIENT_ID = "5728aa1fc9077f7c32000186"
 
-class API:
+
+class API:  # pylint: disable=too-many-instance-attributes
     API_URL = URL("https://graph.tractive.com/3/")
-    CLIENT_ID = "5728aa1fc9077f7c32000186"
+
     DEFAULT_TIMEOUT = 10
 
     TOKEN_URI = "auth/token"
 
-    BASE_HEADERS = {
-        "x-tractive-client": CLIENT_ID,
-        "content-type": "application/json;charset=UTF-8",
-        "accept": "application/json, text/plain, */*",
-    }
-
     def __init__(  # pylint: disable=too-many-arguments
-        self, login, password, timeout=DEFAULT_TIMEOUT, loop=None, session=None
+        self,
+        login,
+        password,
+        client_id=CLIENT_ID,
+        timeout=DEFAULT_TIMEOUT,
+        loop=None,
+        session=None,
     ):
         self._login = login
         self._password = password
+        self._client_id = client_id
         self._timeout = timeout
 
         self.session = session
@@ -42,12 +46,12 @@ class API:
         self._auth_headers = None
 
     async def user_id(self):
-        await self._authenticate()
+        await self.authenticate()
         return self._user_credentials["user_id"]
 
     async def auth_headers(self):
-        await self._authenticate()
-        return {**self.BASE_HEADERS, **self._auth_headers}
+        await self.authenticate()
+        return {**self.base_headers(), **self._auth_headers}
 
     async def request(self, *args, **kwargs):
         """Perform request with error wrapping."""
@@ -76,9 +80,12 @@ class API:
                 return await response.json()
             return await response.read()
 
-    async def _authenticate(self):
+    async def authenticate(self):
         """Perform authenticateion."""
-        # TODO: update credentials if expired
+        if self._user_credentials is not None and self._user_credentials["expires_at"] - time.time() < 3600:
+            self._user_credentials = None
+            self._auth_headers = None
+
         if self._user_credentials is not None:
             return self._user_credentials
 
@@ -93,7 +100,7 @@ class API:
                         "grant_type": "tractive",
                     }
                 ),
-                headers=self.BASE_HEADERS,
+                headers=self.base_headers(),
                 timeout=self._timeout,
             ) as response:
                 try:
@@ -117,3 +124,10 @@ class API:
         """Close the session."""
         if self.session and self._close_session:
             await self.session.close()
+
+    def base_headers(self):
+        return {
+            "x-tractive-client": self._client_id,
+            "content-type": "application/json;charset=UTF-8",
+            "accept": "application/json, text/plain, */*",
+        }
